RegionAllocator

Struct RegionAllocator 

pub struct RegionAllocator { /* private fields */ }

Implementations

Source

impl RegionAllocator

pub fn project<'__pin>( self: Pin<&'__pin mut Self>, ) -> RegionAllocatorProjection<'__pin>

Pin-projects all fields of Self.

These fields are structurally pinned:

  • mu
  • allocated_regions_by_base
  • avail_regions_by_base
  • avail_regions_by_size

These fields are not structurally pinned:

  • region_pool
Source

impl RegionAllocator

pub fn lock_mu(&self) -> impl PinInit<RegionAllocatorMuGuard<'_>, Infallible>

pub fn lock_mu_policy<P: LockPolicy<RawMutex, AcquireArgs = ()>>( &self, ) -> impl PinInit<RegionAllocatorMuGuard<'_, P>, Infallible>

pub fn lock_mu_aliased<'a, AliasClass: LockClass, M2: RawLock>( &'a self, alias: &'a KMutex<AliasClass, M2>, ) -> impl PinInit<KMutexAliasedGuard<'a, RegionAllocatorMuClass, AliasClass, RawMutex, M2>, Infallible>

pub fn lock_mu_aliased_policy<'a, AliasClass: LockClass, M2: RawLock, P: LockPolicy<RawMutex, AcquireArgs = ()>>( &'a self, alias: &'a KMutex<AliasClass, M2>, ) -> impl PinInit<KMutexAliasedGuard<'a, RegionAllocatorMuClass, AliasClass, RawMutex, M2, P>, Infallible>

pub fn guard_mu<'b, 'a>( &'b self, token: &'b LockToken<'a, RegionAllocatorMuClass>, ) -> RegionAllocatorMuTokenGuard<'b, 'a>

pub fn guard_mu_mut<'b, 'a>( &'b self, token: &'b mut LockToken<'a, RegionAllocatorMuClass>, ) -> RegionAllocatorMuTokenGuardMut<'b, 'a>

Source

impl RegionAllocator

pub const REGION_POOL_SLAB_SIZE: usize = 4096

pub fn init() -> impl PinInit<Self, Infallible>

pub fn init_with_pool( pool: RefPtr<RegionPool>, ) -> impl PinInit<Self, Infallible>

Initialize a RegionAllocator with a specified RegionPool.

pub fn has_region_pool(&self) -> bool

pub fn set_region_pool(&self, pool: RefPtr<RegionPool>) -> Result<(), Status>

pub fn reset(&self)

pub fn add_region( &self, region: RegionSpan, allow_overlap: AllowOverlap, ) -> Result<(), Status>

pub fn subtract_region( &self, to_subtract: RegionSpan, allow_incomplete: AllowIncomplete, ) -> Result<(), Status>

pub fn get_region( &self, size: u64, alignment: u64, ) -> Result<UniquePtr<Region>, Status>

pub fn get_region_pointer_aligned( &self, size: u64, ) -> Result<UniquePtr<Region>, Status>

Get a region out of the set of currently available regions which has a specified size and is pointer-aligned (aligned to core::mem::size_of::<*const ()>()).

pub fn get_region_specific( &self, requested_region: RegionSpan, ) -> Result<UniquePtr<Region>, Status>

pub fn test_region_intersects( &self, region: RegionSpan, which: TestRegionSet, ) -> Result<bool, Status>

pub fn test_region_contained_by( &self, region: RegionSpan, which: TestRegionSet, ) -> Result<bool, Status>

pub fn allocated_region_count(&self) -> usize

pub fn available_region_count(&self) -> usize

pub fn walk_allocated_regions<F>(&self, cb: F)
where F: FnMut(&Region) -> bool,

Walk the allocated regions and call the user provided callback for each entry. Stop when out of entries or the callback returns false.

Warning

It is absolutely required that the user callback must not call into any other RegionAllocator public APIs, and should likely not acquire any locks of any kind. This method cannot protect against deadlocks and lock inversions that are possible by acquiring the allocation lock before calling the user provided callback. Because KMutex is not recursive, calling back into the allocator from within the callback will deadlock.

pub fn walk_available_regions<F>(&self, cb: F)
where F: FnMut(&Region) -> bool,

Walk the available regions and call the user provided callback for each entry. Stop when out of entries or the callback returns false.

Warning

It is absolutely required that the user callback must not call into any other RegionAllocator public APIs, and should likely not acquire any locks of any kind. This method cannot protect against deadlocks and lock inversions that are possible by acquiring the allocation lock before calling the user provided callback. Because KMutex is not recursive, calling back into the allocator from within the callback will deadlock.